﻿using System;
using System.Windows.Forms;
using System.Dynamic;
using System.Reflection;
using Westwind.Utilities;


/*******************************************************************************************************************
** Projet LateBindingAppDemo:LateBindingApp
** Mesquite IT. LLC. 03/26/2014
**
** Module Name..............: Latebinding
** Date.....................: 03/25/2015
** Author...................: Claude Nikula
** Compiler.................: .Net 4.5.2  C# 6
**
** Description..............: Demo for late binding Class Library's
**
** Return...................: N/A
**
** Requirements.............: 
**                          1. Westwind.Utilities.DLL  http://www.west-wind.com/westwindToolkit/
**                             Help file               http://www.west-wind.com/westwindToolkit/docs/_4ij04e9zw.htm
** 
**                          2. VFPToolkitNET.DLL       http://foxcentral.net/microsoft/vfptoolkitnet.htm
**
** Notes....................: For this demo you will need to drop VFPToolkitNET.DLL bin\debug 
**                            folder.
**
**                            Westwind.Utilities.DLL Install using NUGet Packages
**
** Example..................:
**
** Parameters...............:
****************************************************************************************************************/


namespace LateBindingApp
{
    
    public partial class frmLatebindingDemo : Form
    {
        

        public frmLatebindingDemo()
        {
            InitializeComponent();
            listResult.Items.Clear();
        }

      
        /// <summary>
        /// Loads an assembly
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdWestWind_Click(object sender, EventArgs e)
        {
            object loResult;

            listResult.Items.Clear();

            /**************************************************************
            * Using Reflection to Loads an assembly given its AssemblyName.
            *
            * You can also use Assembly.LoadFrom Method (String)
            * Loads an assembly given its file name or path.
            *************************************************************/
            Assembly VFPAssembly;
            VFPAssembly = Assembly.Load("VFPToolkitNET");



            //Creates an instance of a type based on a string. Assumes that the type's
            object obj = ReflectionUtils.CreateInstanceFromString("VFPToolkit.strings");


            /*************************************************************
            * If we were not able to load assembly then retry by
            * using Appdomain.Load()
            *
            * Represents an application domain, which is an isolated 
            * environment where applications execute.
            *
            * Load() method - Loads an Assembly given its AssemblyName.
            **************************************************************/
            if (obj == null)
            {
                try
                {
                    var ass = AppDomain.CurrentDomain.Load("VFPToolkit.strings");
                    
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error" + ex.Message);
                }
            }
            else
            {
                try
                {
                    /*****************************************************
                    * Calls a method on an object dynamically. This version 
                    * requires explicit specification of the parameter type 
                    * signatures.
                    *****************************************************/
                    loResult = ReflectionUtils.CallMethod(obj, "Asc", Convert.ToChar("#"));
                    

                    #region loResult fluff 1
                    if (loResult != null)
                    {
                        listResult.Items.Add("Asc Value for # Is: " + loResult);
                    }
                    


                    loResult = ReflectionUtils.CallMethod(obj, "Asc", Convert.ToChar(" "));
                    if (loResult != null)
                    {
                        listResult.Items.Add("Asc Value for spaces Is: " + loResult);
                    }

                    listResult.Items.Add("________________________________");
                    listResult.Items.Add("Wright a string out to a file name Demo.txt");
                  

                    ReflectionUtils.CallMethod(obj, "StrToFile", "This is a demo for the Como .Net User Group", "C:\\Temp\\Demo.TXT");
                    loResult = ReflectionUtils.CallMethod(obj, "FileToStr", "C:\\Temp\\Demo.TXT");
                    if (loResult != null)
                    {
                        listResult.Items.Add("File contained the following");
                        listResult.Items.Add(loResult.ToString());
                    }
                    else
                    {
                        listResult.Items.Add("Was not able to read file");
                    }
                    #endregion

                }
                catch (Exception ex)
                {

                    MessageBox.Show("Error" + ex.Message);
                }
            }
        }

        private void cmdReflection_Click(object sender, EventArgs e)
        {

            try
            {
                /**************************************************************
               * Using Reflection to Loads an assembly given its AssemblyName.
               *
               * You can also use Assembly.LoadFrom Method (String)
               * Loads an assembly given its file name or path.
               *************************************************************/
                Assembly VFPAssembly;
                VFPAssembly = Assembly.Load("VFPToolkitNET");

                // Get the type to use.
                Type _mystringClass = VFPAssembly.GetType("VFPToolkit.strings");

                // Get the method to call.
                MethodInfo myMethodASC = _mystringClass.GetMethod("Asc");

                // Create an instance.
                object obj = Activator.CreateInstance(_mystringClass);

               // Execute the method.
                object loResult = myMethodASC.Invoke(obj, new object[] { Convert.ToChar("#") });

                #region hide fluff
                listResult.Items.Add("________________________________");
                listResult.Items.Add("");
                listResult.Items.Add("Repeating Demo Using Reflection");

                if (loResult != null)
                {
                    listResult.Items.Add("Asc Value for # Is: " + loResult);
                }
                #endregion


                // Execute the method.
                loResult = null;        //Reset
                loResult = myMethodASC.Invoke(obj, new object[] { Convert.ToChar(" ") });

                #region Hide more fluff
                if (loResult != null)
                {
                    listResult.Items.Add("Asc Value for spaces Is: " + loResult);
                }
                #endregion
                //
                //
                //
                //

               #region hide even more fluff

                listResult.Items.Add("");
                listResult.Items.Add("Read a file to a string. File name Demo.txt");

                #endregion

                // Get the type to use.
                Type myStringClass = VFPAssembly.GetType("VFPToolkit.strings");

                // Get the method to call.
                MethodInfo myMethodFileToStr = myStringClass.GetMethod("FileToStr");

                
                // Create an instance.
                obj = Activator.CreateInstance(myStringClass);


                // Execute the method.
                loResult = null;        //Reset
                loResult = myMethodFileToStr.Invoke(obj, new object[] { "C:\\Temp\\Demo.TXT" });


                #region Hide list result fluff
                listResult.Items.Add("String result is: " + loResult);
                #endregion
            }
            catch (Exception ex)
            {

                MessageBox.Show("Error" + ex.Message); 
            }

           
        }

        private void cmdWestWindProp_Click(object sender, EventArgs e)
        {
            listResult.Items.Clear();
            listResult.Items.Add("Syntax for getting property when using Reflection Westwind utilities");
            listResult.Items.Add("string stringVal = ReflectionUtils.GetProperty(obj,''SomeProperty'')");

            listResult.Items.Add("");
            listResult.Items.Add("Syntax for setting property when using Reflection Westwind utilities");
            listResult.Items.Add("ReflectionUtils.SetProperty(obj, ''string property name'', object value);");

            //ParameterInfo[] Param = myMethodASC.GetParameters();
            //Param.GetValue("ParamName");
            //Param.SetValue("ParamName", "Vulae");
        }

        private void cmdDynamic_Click(object sender, EventArgs e)
        {
            try
            {
                /**********************************************************
                * C# version 4 or grader 
                * Simplifying Late-Bound Calls Using Dynamic Types
                * Note: Be sure to have a Ref set to Microsoft.CSharp.dll
                *       when using the Dynamic option 
                *
                *      Dynamic Does not seem to work with static types
                *********************************************************/
                Assembly VFPAssembly;
                VFPAssembly = Assembly.Load("VFPToolkitNET");

                // Get metadata for the strings type (class)
                dynamic myStringClass = VFPAssembly.GetType("VFPToolkit.strings");

                //Create the strings class on the fly and call Asc method!
                dynamic obj = Activator.CreateInstance(myStringClass);
                //dynamic dynamicResult = obj.Asc(Convert.ToChar("#"));

                dynamic myMethodAsc = myStringClass.GetMethod("Asc");
                dynamic dynamicResult = myMethodAsc.Invoke(obj, new object[] {Convert.ToChar("#")});


                listResult.Items.Add(" ");
                listResult.Items.Add("____________________________");
                listResult.Items.Add("Late Binding Using Dynamic");
                listResult.Items.Add("Asc Value for # is: " + dynamicResult);
                listResult.Items.Add(" ");
                listResult.Items.Add("Note: Be sure to have a Ref set to Microsoft.CSharp.dll when using the Dynamic option Dynamic Does not seem to work with static types ");
            }
            catch (Exception ex)
            {

                MessageBox.Show("Error" + ex.Message);
            }
            
        }
    }
}
